---
title: Project.swift
description: 'This page documents the models that users can use to define their project: how to initialize them, attributes and their meaning, protocol comformances.'
slug: '/manifests/project'
---

Projects are defined in `Project.swift` files, which we refer to as manifest files. Manifests need to import the framework `ProjectDescription` which contains all the classes and enums that are available for you to describe your projects. The snippet below shows an example project manifest:

```swift

let project = Project(
    name: "MyProject",
    organizationName: "MyOrg",
    targets: [
        Target(
            name: "App",
            platform: .iOS,
            product: .app,
            bundleId: "io.tuist.App",
            infoPlist: "Config/App-Info.plist",
            sources: ["Sources/**"],
            resources: [
                "Resources/**",
                .folderReference(path: "Stubs"),
                .folderReference(path: "ODR", tags: ["odr_tag"])
            ],
            headers: Headers(
                public: ["Sources/public/A/**", "Sources/public/B/**"],
                private: "Sources/private/**",
                project: ["Sources/project/A/**", "Sources/project/B/**"]
            ),
            dependencies: [
                .project(target: "Framework1", path: "../Framework1"),
                .project(target: "Framework2", path: "../Framework2")
            ]
        )
    ],
    schemes: [
        Scheme(
            name: "App-Debug",
            shared: true,
            buildAction: .buildAction(targets: ["App"]),
            testAction: .targets(["AppTests"]),
            runAction: .runAction(executable: "App")
        ),
        Scheme(
            name: "App-Release",
            shared: true,
            buildAction: .buildAction(targets: ["App"]),
            runAction: .runAction(executable: "App")
        )
    ],
    additionalFiles: [
        "Dangerfile.swift",
        "Documentation/**",
        .folderReference(path: "Website")
    ]
)
```

### Project

A `Project.swift` should initialize a variable of type `Project`. It can take any name, although we recommend to stick to `project`. A project accepts the following attributes:

| Property               | Description                                                                                                                                                                                                     | Type                                             | Required | Default    |
| ---------------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | ------------------------------------------------ | -------- | ---------- |
| `name`                 | Name of the project. It's used to determine the name of the generated Xcode project.                                                                                                                            | `String`                                         | Yes      |            |
| `organizationName`     | Name of the organization used by Xcode as copyright when generating new files.                                                                                                                                  | `String`                                         | No       |            |
| `options`              | Additional project options                                                                                                                                                                                      | [`[ProjectOption]`](#project-options)            | No       | `[]`       |
| `packages`             | List of Swift Packages used by the project.                                                                                                                                                                     | [`[Package]`](#package)                          | No       | `[]`       |
| `settings`             | Project build settings and configuration files                                                                                                                                                                  | [`Settings`](#settings)                          | No       | `[]`       |
| `targets`              | List of targets within the project.                                                                                                                                                                             | [`[Target]`](#target)                            | Yes      | `[]`       |
| `schemes`              | List of custom schemes within the project, default scheme for each target is generated by default (see The [Configuration `generationOptions`](manifests/config.md#generationoption) to configure this behavior) | [`[Scheme]`](#scheme)                            | No       | `[]`       |
| `fileHeaderTemplate`   | Lets you define custom file header template macro for built-in Xcode file templates.                                                                                                                            | [`FileHeaderTemplate`](#file-header-template)    | No       |            |
| `additionalFiles`      | List of project related files to include in the project - these won't be included in any of the build phases. For target related files, use the corresponding `Target.additionalFiles` parameter.               | [`[FileElement]`](#fileelement)                  | No       | `[]`       |
| `resourceSynthesizers` | List of resource synthesizer for generating accessors for resources.                                                                                                                                            | [`[ResourceSynthesizer]`](#resource-synthesizer) | No       | `.default` |

### Project Options
List of additional options:

| Case       | Description                                                   |
| ---------- | ------------------------------------------------------------- |
| `.textSettings(usesTabs: Bool?, indentWidth: UInt?, tabWidth: UInt?, wrapsLines: Bool?)`| [Text settings](#text-settings) to override user ones for current project. |

#### Text Settings
Text settings to override user ones. Would use Xcode defined settings if pass `nil`.

| Property      | Description          | Type    | Required | Default |
| ------------- | -------------------- | ------- | -------- | ------- |
| `usesTabs`    | Use tabs over spaces | `Bool?` | No       | `nil`   |
| `indentWidth` | Indent width         | `UInt?` | No       | `nil`   |
| `tabWidth`    | Tab width            | `UInt?` | No       | `nil`   |
| `wrapsLines`  | Wrap lines           | `Bool?` | No       | `nil`   |

### Package

You can add Swift Packages very similarly to how you add dependencies in a `Package.swift`:

For remote:

```swift
.package(url: "https://github.com/tuist/XcodeProj", .exact("6.7.0"))
```

```swift
.package(url: "https://github.com/tuist/XcodeProj", from: "6.7.0")
```

```swift
.package(url: "https://github.com/tuist/XcodeProj", "6.6.1"..<"6.8.0")
```

For local:

```swift
.package(path: "MyLibrary")
```

Targets can then depend on products from these Swift Packages. See [Dependencies](guides/dependencies.md)

:::note Package.resolved
Tuist creates a `.package.resolved` file, so your team can share the same versions of dependencies without committing your workspace.
:::

### Target

Each target in the list of project targets can be initialized with the following attributes:

| Property           | Description                                                                                                                                                                                        | Type                                            | Required | Default          |
| ------------------ | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | ----------------------------------------------- | -------- | ---------------- |
| `name`             | The name of the target. The Xcode project target and the derivated product take the same name.                                                                                                     | `String`                                        | Yes      |                  |
| `platform`         | The platform the target product is built for.                                                                                                                                                      | [`Platform`](#platform)                         | Yes      |                  |
| `product`          | The type of build product this target will output.                                                                                                                                                 | [`Product`](#product)                           | Yes      |                  |
| `productName`      | The built product name.                                                                                                                                                                            | `String`                                        | No       | `$(TARGET_NAME)` |
| `bundleId`         | The product bundle identifier.                                                                                                                                                                     | `String`                                        | Yes      |                  |
| `deploymentTarget` | The minimum iOS version your product will support.                                                                                                                                                 | [`DeploymentTarget`](#deployment-target)        | No       |                  |
| `infoPlist`        | Relative path to the `Info.plist`                                                                                                                                                                  | [`InfoPlist`](#infoplist)                       | No       | `.default`       |
| `sources`          | Source files that are compiled by the target. Any playgrounds matched by the globs used in this property will be automatically added.                                                              | [`SourceFilesList`](#source-file-list)          | Yes      |                  |
| `resources`        | List of files to include in the resources build phase. Note that localizable files, `*.lproj`, are supported.                                                                                      | [`ResourceFileElements`](#resourcefileelements) | No       |                  |
| `copyFiles`        | Copy files actions allow defining copy files build phases.                                                                                                                                         | [`[CopyFilesAction]`](#copy-files-action)       | No       |                  |
| `headers`          | The target headers.                                                                                                                                                                                | [`Headers`](#headers)                           | No       |                  |
| `entitlements`     | Path to the entitlement file.                                                                                                                                                                      | [`Path`](#path)                                 | No       |                  |
| `scripts`          | Target scripts allow defining extra script build phases.                                                                                                                                           | [`[TargetScript]`](#target-scripts)              | No       | `[]`             |
| `dependencies`     | List of target dependencies.                                                                                                                                                                       | [`[TargetDependency]`](guides/dependencies.md)   | No       | `[]`             |
| `settings`         | Target build settings and configuration files.                                                                                                                                                     | [`Settings`](#settings)                         | No       |                  |
| `coreDataModels`   | Core Data models.                                                                                                                                                                                  | [`[CoreDataModel]`](#core-data-model)           | No       |                  |
| `environment`      | List of variables that will be set to the scheme that Tuist automatically generates for the target.                                                                                                | `[String: String]`                              | No       | `[:]`            |
| `launchArguments`  | List of launch arguments that will be set to the scheme that Tuist automatically generates for the target.                                                                                         | [`[LaunchArgument]`](#launch-argument)          | No       | `[]`             |
| `additionalFiles`  | List of target related files to include in the project - these won't be included in any of the build phases. For project related files, use the corresponding `Project.additionalFiles` parameter. | [`[FileElement]`](#fileelement)                 | No       | `[]`             |

```swift
import ProjectDescription

let target = Target(name: "App",
                    platform: .iOS,
                    product: .app,
                    bundleId: "io.tuist.App",
                    infoPlist: "Info.plist",
                    sources: "Sources/**",
                    dependencies: [])
```

### Source file list

It represents a list of source files that are part of a target:

| Property | Description                                                                                                                              | Type                                    | Required | Default |
| -------- | ---------------------------------------------------------------------------------------------------------------------------------------- | --------------------------------------- | -------- | ------- |
| `globs`  | Path to the source files. They are represented by a [glob pattern](https://facelessuser.github.io/wcmatch/glob/) and the compiler flags. | [`[SourceFileGlob]`](#source-file-glob) | No       |         |

:::note ExpressibleByStringLiteral and ExpressibleByArrayLiteral
The list of source files can be initialized with a string that represents the glob pattern, or an array of strings, which represents a list of glob patterns. In both cases the comiler flags will have no value.
:::

:::note Patterns matching the same paths
If multiple patterns match the same paths, the latest one takes preference over the others. That also means that the latest compiler flags will be applied.
:::

#### Source file glob

It represents a glob pattern that refers to source files and the compiler flags _(if any)_ to be set in the build phase:

| Property        | Description                                                                  | Type              | Required | Default |
| --------------- | ---------------------------------------------------------------------------- | ----------------- | -------- | ------- |
| `glob`          | Glob pattern to the source files.                                            | [`Path`](#path)   | Yes      |         |
| `excluding`     | Glob patterns for source files that will be excluded.                        | [`[Path]`](#path) | No       | `[]`    |
| `compilerFlags` | The compiler flags to be set to the source files in the sources build phase. | `String`          | No       |         |
| `codeGen`       | The source file attribute to be set in the build phase.                      | [`FileCodeGen`](#FileCodeGen)|   No      |       |

### Copy Files Action

Copy files actions, represented as target copy files build phases, are useful to associate project files and products of other targets with the target and copies them to a specified destination, typically a subfolder within a product. This action may be used multiple times per target.

| Case                                                                       | Description                                              |
| -------------------------------------------------------------------------- | -------------------------------------------------------- |
| `.productsDirectory(name: String, subpath: String?, files: [FileElement])` | A copy files action for the products directory.          |
| `.wrapper(name: String, subpath: String?, files: [FileElement])`           | A copy files action for the wrapper directory.           |
| `.executables(name: String, subpath: String?, files: [FileElement])`       | A copy files action for the executables directory.       |
| `.resources(name: String, subpath: String?, files: [FileElement])`         | A copy files action for the resources directory.         |
| `.javaResources(name: String, subpath: String?, files: [FileElement])`     | A copy files action for the java resources directory.    |
| `.frameworks(name: String, subpath: String?, files: [FileElement])`        | A copy files action for the frameworks directory.        |
| `.sharedFrameworks(name: String, subpath: String?, files: [FileElement])`  | A copy files action for the shared frameworks directory. |
| `.sharedSupport(name: String, subpath: String?, files: [FileElement])`     | A copy files action for the shared support directory.    |
| `.plugins(name: String, subpath: String?, files: [FileElement])`           | A copy files action for the plugins directory.           |

The following example shows the definition of a copy files action that copy template files to the shared support directory and adds it inside a subfolder `Templates`:

```swift
.sharedSupport(name: "Copy Templates", subpath: "Templates", files: "MyTemplates/**")
```

The following example shows the definition of a copy files action that copy font files to the resources directory and adds it inside a subfolder `Fonts`:

```swift
.resources(name: "Copy Fonts", subpath: "Fonts", files: "MyFonts/**")
```

### Headers

It represents the target headers:

| Property           | Description                                                                           | Type                                                     | Required | Default |
| ------------------ | ---------------------------------------------------------------------------------------- | ----------------------------------------------------- | -------- | ------- |
| `public`           | Relative glob pattern that points to the public headers.                                 | [`FileList`](#file-list)                              | No       |         |
| `private`          | Relative glob pattern that points to the private headers.                                | [`FileList`](#file-list)                              | No       |         |
| `project`          | Relative glob pattern that points to the project headers.                                | [`FileList`](#file-list)                              | No       |         |
| `exclusionRule`    | Rule, which determines how to resolve found duplicates in public/private/project scopes  | [`AutomaticExclusionRule`](#automatic-exclusion-rule) | No       | `.none` |

### File List

It represents a list of glob patterns that refer to files:

| Property | Description                | Type                                | Required | Default |
| -------- | -------------------------- | ----------------------------------- | -------- | ------- |
| `globs`  | Glob pattern to the files. | [`[FileListGlob]`](#file-list-glob) | Yes      |         |

:::note ExpressibleByStringLiteral and ExpressibleByArrayLiteral
The list of files can be initialized with a string that represents the glob pattern, or an array of strings, which represents a list of glob patterns.
:::

#### File list glob

It represents a glob pattern that refers to files:

| Property        | Description                                                                  | Type              | Required | Default |
| --------------- | ---------------------------------------------------------------------------- | ----------------- | -------- | ------- |
| `glob`          | Glob pattern to the files.                                                   | [`Path`](#path)   | Yes      |         |
| `excluding`     | Glob patterns for source files that will be excluded.                        | [`[Path]`](#path) | No       | `[]`    |

### Automatic Exclusion Rule

Determine how to resolve cases, when the same files found in different header scopes

| Case                              | Description                                                                                                                             |
| --------------------------------- | --------------------------------------------------------------------------------------------------------------------------------------- |
| `none`                            | Nothing to do, should be resolved manually                                                                                              |
| `projectExcludesPrivateAndPublic` | All found `public` headers will be exluded from `private` scope, then `public` and `private` will be excluded from `project`            |
| `publicExcludesPrivateAndProject` | Reversed. All found `project` headers will be exluded from `private` scope, then `project` and `private` will be excluded from `public` |

### Core Data Model

The `CoreDataModel` type represents a Core Data model:

| Property         | Description                            | Type            | Required | Default |
| ---------------- | -------------------------------------- | --------------- | -------- | ------- |
| `path`           | Relative path to the Core Data model.  | [`Path`](#path) | Yes      |         |
| `currentVersion` | Current version without the extension. | `String`        | Yes      |         |

### Platform

The platform type represents the platform a target is built for. It can be any of the following types:

| Case       | Description                                                   |
| ---------- | ------------------------------------------------------------- |
| `.iOS`     | The [iOS](https://en.wikipedia.org/wiki/IOS) platform         |
| `.macOS`   | The [macOS](https://en.wikipedia.org/wiki/MacOS) platform     |
| `.tvOS`    | The [tvOS](https://en.wikipedia.org/wiki/TvOS) platform       |
| `.watchOS` | The [watchOS](https://en.wikipedia.org/wiki/WatchOS) platform |

### Product

The type of build product this target will output. It can be any of the following types:

| Case                    | Description                                                           |
| ----------------------- | --------------------------------------------------------------------- |
| `.app`                  | An application.                                                       |
| `.staticLibrary`        | A static library.                                                     |
| `.dynamicLibrary`       | A dynamic library.                                                    |
| `.framework`            | A dynamic framework.                                                  |
| `.staticFramework`      | A static framework.                                                   |
| `.unitTests`            | A unit tests bundle.                                                  |
| `.uiTests`              | A UI tests bundle.                                                    |
| `.bundle`               | A custom bundle. (currently only iOS resource bundles are supported). |
| `.appExtension`         | An application extension.                                             |
| `.stickerPackExtension` | A sticker pack extension.                                             |
| `.watch2App`            | A Watch application. (watchOS platform only) .                        |
| `.watch2Extension`      | A Watch application extension. (watchOS platform only).               |
| `.messagesExtension`    | An iMessage extension. (iOS platform only)                            |
| `.appClip`              | An appClip. (iOS platform only).                                      |
| `.commandLineTool`      | A command line tool (macOS platform only).                            |

### InfoPlist

The `InfoPlist` model represents a target `Info.plist` file. It can have any of the following values:

| Case                                                 | Description                                                                                                                     |
| ---------------------------------------------------- | ------------------------------------------------------------------------------------------------------------------------------- |
| `.file(path: Path)`                                  | The path to an existing Info.plist file.                                                                                        |
| `.dictionary([String: InfoPlist.Value])`             | A dictionary with the Info.plist content. Tuist generates the Info.plist file at the generation time.                           |
| `.default`                                           | Tells Tuist to provide the default content for the target the InfoPlist belongs to.                                             |
| `.extendingDefault(with: [String: InfoPlist.Value])` | Tells Tuist to provide the default content for the target the InfoPlist belongs to, and extend it with the given values.        |

:::note ExpressibleByStringLiteral
The InfoPlist model conforms the ExpressibleByStringLiteral protocol, which means that it can be initialized with a String. In that case, the string is the path to the Info.plist file.
:::

### InfoPlist.Value

It represents the values of the InfoPlist file dictionary. The reason this type exists is to ensure that the values used to define the content of the dynamically generated `Info.plist` files are valid:

| Case                                     | Description                       |
| ---------------------------------------- | --------------------------------- |
| `.string(String)`                        | It represents a string value.     |
| `.integer(Int)`                          | It represents an integer value.   |
| `.real(Double)`                          | It represents a floating value.   |
| `.boolean(Bool)`                         | It represents a boolean value.    |
| `.dictionary([String: InfoPlist.Value])` | It represents a dictionary value. |
| `.array([InfoPlist.Value])`              | It represents an array value.     |

:::note ExpressiveByLiteral
`InfoPlist.Value` conforms to the `ExpressiveByLiteral` protocols and therefore, it can be initialized with an instance of the primitive type that they encapsulate.
:::

### Target Scripts

Target scripts, represented as target script build phases in the generated Xcode projects, are useful to define actions to be executed before of after the build process of a target.

| Case                                                                                                                                                                                         | Description                                                                                                                                                       |
| -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | ----------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `.pre(tool: String, arguments: String..., name: String, inputPaths: [Path], inputFileListPaths: [Path], outputPaths: [Path], outputFileListPaths: [Path], basedOnDependencyAnalysis: Bool, runForInstallBuildsOnly: Bool, shellPath: String)`  | Script executed before the target-specific build phases where tool is the name of the tool to be executed.                                                        |
| `.pre(path: Path, arguments: String..., name: String, inputPaths: [Path], inputFileListPaths: [Path], outputPaths: [Path], outputFileListPaths: [Path], basedOnDependencyAnalysis: Bool, runForInstallBuildsOnly: Bool, shellPath: String)`    | Script executed before the target-specific build phases where path is the path to the tool to be executed.                                                        |
| `.pre(script: String, name: String, inputPaths: [Path], inputFileListPaths: [Path], outputPaths: [Path], outputFileListPaths: [Path], basedOnDependencyAnalysis: Bool, runForInstallBuildsOnly: Bool, shellPath: String)`                      | Script executed before the target-specific build phases where script is an embedded script to run. It is advised to keep embedded scripts as small as possible.                              |
| `.post(tool: String, arguments: String..., name: String, inputPaths: [Path], inputFileListPaths: [Path], outputPaths: [Path], outputFileListPaths: [Path], basedOnDependencyAnalysis: Bool, runForInstallBuildsOnly: Bool, shellPath: String)` | Script executed after all the target-specific build phases where tool is the name of the tool to be executed.                                                     |
| `.post(path: Path, arguments: String..., name: String, inputPaths: [Path], inputFileListPaths: [Path], outputPaths: [Path], outputFileListPaths: [Path], basedOnDependencyAnalysis: Bool, runForInstallBuildsOnly: Bool, shellPath: String)`   | Script executed after all the target-specific build phases where path is the path to the tool to be executed.                                                     |
| `.post(script: String, name: String, inputPaths: [Path], inputFileListPaths: [Path], outputPaths: [Path], outputFileListPaths: [Path], basedOnDependencyAnalysis: Bool, runForInstallBuildsOnly: Bool, shellPath: String)`                     | Script executed after all the target-specific build phases where script is an embedded script to run. It is advised to keep embedded scripts as small as possible |

The following example shows the definition of an script that runs the `my_custom_script.sh` passing the argument `"hello"`:

```swift
.pre(path: "my_custom_script.sh", arguments: ["hello"], name: "My Custom Script Phase")
```

The following example shows the definition of an script that runs the `my_custom_script.sh` passing the argument `"hello"` on the `/bin/zsh` shell (default is `/bin/sh`):

```swift
.pre(path: "my_custom_script.sh", arguments: ["hello"], name: "My Custom Script Phase", shellPath: "/bin/zsh")
```

The following example shows the definition of an script that runs the `my_custom_script.sh` defining input and output files:

```swift
.pre(path: "my_custom_script.sh", name: "My Custom Script Phase", inputFileListPaths: [ "Data/Cars.raw.json", "Data/Drivers.raw.json" ], outputFileListPaths: [ "Data/Cars.swift", "Data/Drivers.swift" ])
```

The following example shows the definition of an script that runs the `my_custom_script.sh` on all incremental builds, regardless of whether or not something has changed:

```swift
.pre(path: "my_custom_script.sh", name: "My Custom Script Phase", basedOnDependencyAnalysis: false)
```

The following example shows the definition of an script that runs the `my_custom_script.sh` only for install builds:

```swift
.pre(path: "my_custom_script.sh", name: "My Custom Script Phase", runForInstallBuildsOnly: true)
```

### Configuration name

`ConfigurationName` is a wrapper around `String` to type the project or workspace configurations. The type provides the `.debug` and `.release` static variables for the `Debug` and `Release` configuration respectively, and we recommend adding new configurations using a extension:

```swift
import ProjectDescription

extension ConfigurationName {
  static var beta: ConfigurationName {
      ConfigurationName("Beta")
  }
}
```

### Scheme

A `Scheme` defines a collection of targets to `Build, Run, Test, Profile, Analyze and Archive`:

| Property        | Description                                                                                                                      | Type                               | Required | Default |
| --------------- | -------------------------------------------------------------------------------------------------------------------------------- | ---------------------------------- | -------- | ------- |
| `name`          | The name of the scheme.                                                                                                          | `String`                           | Yes      |         |
| `shared`        | Marks the scheme as shared (i.e. one that is checked in to the repository and is visible to `xcodebuild` from the command line). | `Bool`                             | No       | `true`  |
| `hidden`        | When `true` the scheme doesn't show up in the dropdown scheme's list.                                                            | `Boolean`                          | No       | `false` |
| `buildAction`   | Action that builds the project targets.                                                                                          | [`BuildAction`](#build-action)     | No       |         |
| `testAction`    | Action that runs the project tests.                                                                                              | [`TestAction`](#test-action)       | No       |         |
| `runAction`     | Action that runs project built products.                                                                                         | [`RunAction`](#run-action)         | No       |         |
| `archiveAction` | Action that runs the project archive.                                                                                            | [`ArchiveAction`](#archive-action) | No       |         |
| `profileAction` | Action that profiles the project.                                                                                                | [`ProfileAction`](#profile-action) | No       |         |
| `analyzeAction` | Action that analyze the project.                                                                                                 | [`AnalyzeAction`](#analyze-action) | No       |         |

:::note Auto-generation of schemes
Tuist will auto-generate a scheme for each target by default in addition to any defined schemes. This however can be disabled if needed via the [Configuration generationOptions](manifests/config.md#generationoption).
:::

:::note Schemes & Configurations
When generating the default target schemes, the 'Debug' configuration is used for the Run, Test and Analyze actions and the 'Release' configuration is used for the Profile & Archive actions. If those configurations aren't specified, the first (alphabetically ordered) custom configuration with the corresponding variant is selected in their place.
:::

:::note Extension Schemes
Schemes for extensions have additional properties and settings, Tuist automatically applies those to any scheme in which the first Target in the Build Action's list of targets is an extension.
:::

:::note Archive, profile & analyze actions definition
Take in count that `ArchiveAction`, `ProfileAction` and `AnalyzeAction` when are defined in a `Scheme` in isolated or in set, needs also a definition of `BuildAction` in the signature of the scheme. Without the previously option commented the scheme will have the these their definitions by default.
:::

#### Build Action

It represents the scheme action that builds targets. It's initialized with the `.buildAction` static method and the following attributes can be passed:

| Property                  | Description                                                            | Type                                     | Required | Default |
| ------------------------- | ---------------------------------------------------------------------- | ---------------------------------------- | -------- | ------- |
| `targets`                 | A list of targets to build, which are defined in the project.          | [`[TargetReference]`](#target-reference) | No       |         |
| `preActions`              | A list of actions that are executed before starting the build process. | [`[ExecutionAction]`](#execution-action) | Yes      | `[]`    |
| `postActions`             | A list of actions that are executed after the build process.           | [`[ExecutionAction]`](#execution-action) | Yes      | `[]`    |
| `runPostActionsOnFailure` | Whether the post actions should be run in the case of a failure        | `Bool`                                   | No       | `false` |

#### Run action

It represents the scheme action that runs the built products on the supported platforms. It's initialized with the `.runAction` static method and the following attributes can be passed:

| Property             | Description                                                          | Type                                                      | Required | Default                |
| -------------------- | -------------------------------------------------------------------- | --------------------------------------------------------- | -------- | ---------------------- |
| `configuration`      | Indicates the build configuration the product should run with.       | [`ConfigurationName`](#configuration-name)                | No       | `.debug`               |
| `attachDebugger`     | Whether a debugger should be attached to the run process or not.     | `Bool`                                                    | No       | `true`                 |
| `preActions`         | A list of actions that are executed before starting the run process. | [`[ExecutionAction]`](#execution-action)                  | Yes      | `[]`                   |
| `postActions`        | A list of actions that are executed after the run process.           | [`[ExecutionAction]`](#execution-action)                  | Yes      | `[]`                   |
| `executable`         | The name of the executable or target to run.                         | [`TargetReference`](#target-reference)                    | No       |                        |
| `arguments`          | Command line arguments passed on launch and environment variables.   | [`Arguments`](#arguments)                                 | No       |                        |
| `options`            | List of options to set to the action.                                | [`RunActionOptions`](#run-action-options)                 | No       | `.options()`           |
| `diagnosticsOptions` | List of diagnostics options to set to the action.                    | [`[SchemeDiagnosticsOption]`](#scheme-diagnostics-option) | No       | `[.mainThreadChecker]` |

#### Test Action

You can create a test action with either a set of test targets or test plans using the `.targets` or `.testPlans` static methods respectively.

When initializing it with a **set of targets**, the attributes you can pass to those methods are documented below:


| Property                   | Description                                                                                                            | Type                                                      | Required | Default                |
| -------------------------- | ---------------------------------------------------------------------------------------------------------------------- | --------------------------------------------------------- | -------- | ---------------------- |
| `targets`                  | A list of testable targets, that are targets which are defined in the project with testable information.               | [`[TestableTarget]`](#testable-target)                    | Yes      | `[]`                   |
| `arguments`                | Command line arguments passed on launch and environment variables.                                                     | [`Arguments`](#arguments)                                 | No       |                        |
| `configuration`            | Build configuration to run the test with.                                                                              | [`ConfigurationName`](#configuration-name)                | No       | `.debug`               |
| `attachDebugger`           | Whether a debugger should be attached to the test process or not.                                                      | `Bool`                                                    | No       | `true`                 |
| `expandVariableFromTarget` | A target that will be used to expand the variables defined inside Environment Variables definition (e.g. $SOURCE_ROOT) | [`TargetReference`](#target-reference)                    | No       |                        |
| `preActions`               | A list of actions that are executed before starting the tests-run process.                                             | [`[ExecutionAction]`](#execution-action)                  | No       | `[]`                   |
| `postActions`              | A list of actions that are executed after the tests-run process.                                                       | [`[ExecutionAction]`](#execution-action)                  | No       | `[]`                   |
| `options`                  | List of options to set to the action.                                                                                  | [`TestActionOptions`](#test-action-options)               | No       | `.options()`           |
| `diagnosticsOptions`       | List of diagnostics options to set to the action.                                                                      | [`[SchemeDiagnosticsOption]`](#scheme-diagnostics-option) | Yes      | `[.mainThreadChecker]` |

When initializing it with a **test plans**, the attributes you can pass to those methods are documented below:

| Property  | Description                      | Type              | Required | Default |
| --------- | -------------------------------- | ----------------- | -------- | ------- |
| `default` | Path to a xctestplan file.       | [`Path`](#path)   | No       |         |
| `other`   | Paths to other xctestplan files. | [`[Path]`](#path) | No       | `[]`    |

:::note Launch Arguments & Environment
By default the Test & Profile actions will inherit the Run action's Launch & Environment when not explicitly specified.
:::

##### Test Action Options

The type `TestActionOptions` represents a set of options to set to configure a test action. The `.options()` static method returns the default options:

| Property              | Description                                                                                             | Type                                     | Required | Default |
| ----------------------| ------------------------------------------------------------------------------------------------------- | -----------------------------------------| -------- | ------- |
| `region`              | Region used to run the tests.                                                                           | `String`                                 | No       |         |
| `language`            | Language used to run the tests.                                                                         | [`SchemeLanguage`](#scheme-language)     | No       |         |
| `coverage`            | Whether the scheme should or not gather the test coverage data.                                         | `Bool`                                   | No       | `false` |
| `codeCoverageTargets` | A list of targets you want to gather the test coverage data for them, which are defined in the project. | [`[TargetReference]`](#target-reference) | No       | `[]`    |

#### Execution Action

Scheme run scripts can be defined with the following attributes:

| Property     | Description                                                                     | Type                                   | Required | Default        |
| ------------ | ------------------------------------------------------------------------------- | -------------------------------------- | -------- | -------------- |
| `title`      | Name of a script.                                                               | `String`                               | No       | `"Run Script"` |
| `scriptText` | An inline shell script.                                                         | [`TargetReference`](#target-reference) | No       |                |
| `target`     | Name of the build or test target that will provide the action's build settings. | `String`                               | No       |                |

#### Testable Target

Testable target describe target and tests information.

| Property                  | Description                           | Type                                   | Required | Default |
| ------------------------- | ------------------------------------- | -------------------------------------- | -------- | ------- |
| `target`                  | The target name and its project path. | [`TargetReference`](#target-reference) | Yes      |         |
| `skipped`                 | Skip test target from TestAction.     | `Bool`                                 | No       | `false` |
| `skipped`                 | Skip test target from TestAction.     | `Bool`                                 | No       | `false` |
| `parallelizable`          | Execute tests in parallel.            | `Bool`                                 | No       | `false` |
| `randomExecutionOrdering` | Execute tests in random order.        | `Bool`                                 | No       | `false` |

#### Arguments

Arguments contain commandline arguments passed on launch and Environment variables.

| Property          | Description                                                                               | Type                                  | Required | Default |
| ----------------- | ----------------------------------------------------------------------------------------- | ------------------------------------- | -------- | ------- |
| `environment`     | The environment variables that are passed by the scheme when running a scheme action.     | `[String: String]`                    | No       | `[:]`   |
| `launchArguments` | Command line launch arguments that are passed by the scheme when running a scheme action. | [[LaunchArgument]`](#launch-argument) | No       | `[]`    |

#### Launch Argument

It represents an argument that is passed when running a scheme's action:

| Property    | Description                            | Type     | Required | Default |
| ----------- | -------------------------------------- | -------- | -------- | ------- |
| `name`      | The name of the launch argument        | `String` | Yes      |         |
| `isEnabled` | Whether the argument is enabled or not | `Bool`   | Yes      |         |

#### Run Action Options

Run action options represent the configuration of a run action.

| Property                    | Description                                                                                                                          | Type                                       | Required | Default |
| --------------------------- | ------------------------------------------------------------------------------------------------------------------------------------ | ------------------------------------------ | -------- | ------- |
| `language`                  | Language to use when running the app.                                                                                                | [`SchemeLanguage`](#scheme-language)                           | No       |         |
| `storeKitConfigurationPath` | Path of the StoreKit Configuration file that is used to control and debug StoreKit purchases. It only works with Xcode 12 and above. | [`Path`](#path)                            | No       |         |
| `simulatedLocation`         | A simulated GPS location to use when running the app.                                                                                | [`SimulatedLocation`](#simulated-location) | No       |         |

#### Simulated Location

Simulated location represents a GPS location that is used when running an app on the simulator.

| Property     | Description                                             | Type            | Required | Default |
| ------------ | ------------------------------------------------------- | --------------- | -------- | ------- |
| `identifier` | The identifier of the location _(e.g. London, England)_ | `String`        | No       |
| `gpxFile`    | Path to a `.gpx` file that indicates the location       | [`Path`](#path) | No       |

`SimulatedLocation` provides convenient initializers for the following locations:

| Location                   | Initializer     |
| -------------------------- | --------------- |
| London, England            | `.london`       |
| Johannesburg, South Africa | `.johannesburg` |
| Moscow, Russia             | `.moscow`       |
| Mumbai, India              | `.mumbai`       |
| Tokyo, Japan               | `.tokyo`        |
| Sydney, Australia          | `.sydney`       |
| Hong Kong, China           | `.hongKong`     |
| Honolulu, HI, USA          | `.honolulu`     |
| San Francisco, CA, USA     | `.sanFrancisco` |
| Mexico City, Mexico        | `.mexicoCity`   |
| New York, NY, USA          | `.newYork`      |
| Rio De Janeiro, Brazil     | `.rioDeJaneiro` |

#### Scheme Diagnostics Option

Diagnostics options represent the configurable diagnostics-related settings in the schemes' run and test actions.

| Case                 | Description                     |
| -------------------- | ------------------------------- |
| `.mainThreadChecker` | Enable the main thread checker. |

#### Archive Action

It represents the scheme action to archive targets' products. It's initialized with the `.archiveAction` static method and the following attributes can be passed:

| Property                   | Description                                                              | Type                                     | Required | Default |
| -------------------------- | ------------------------------------------------------------------------ | ---------------------------------------- | -------- | ------- |
| `configurationName`        | Indicates the build configuration to run the archive with.               | `String`                                 | Yes      |         |
| `revealArchiveInOrganizer` | If set to true, Xcode will reveal the Organizer on completion.           | `Bool`                                   | No       | Yes     |
| `customArchiveName`        | Set if you want to override Xcode's default archive name.                | `String`                                 | No       |         |
| `preActions`               | A list of actions that are executed before starting the archive process. | [`[ExecutionAction]`](#execution-action) | No       | `[]`    |
| `postActions`              | A list of actions that are executed after the archive process.           | [`[ExecutionAction]`](#execution-action) | No       | `[]`    |

#### Profile Action

It represents the scheme action that profiles the built products on the supported platforms. It's initialized with the `.profileAction` static method and the following attributes can be passed:

| Property        | Description                                                              | Type                                       | Required | Default    |
| --------------- | ------------------------------------------------------------------------ | ------------------------------------------ | -------- | ---------- |
| `configuration` | Indicates the build configuration the product should be profiled with.   | [`ConfigurationName`](#configuration-name) | No       | `.release` |
| `preActions`    | A list of actions that are executed before starting the profile process. | [`[ExecutionAction]`](#execution-action)   | Yes      | `[]`       |
| `postActions`   | A list of actions that are executed after the profile process.           | [`[ExecutionAction]`](#execution-action)   | Yes      | `[]`       |
| `executable`    | The name of the executable or target to profile.                         | [`TargetReference`](#target-reference)     | No       |            |
| `arguments`     | Command line arguments passed on launch and environment variables.       | [`Arguments`](#arguments)                  | No       |            |

:::note Launch Arguments & Environment
By default the Test & Profile actions will inherit the Run action's Launch & Environment when not explicitly specified.
:::

#### Analyze Action

It represents the scheme action that analyzes the built products. It's initialized with the `.analyzeAction` static method and the following attributes can be passed:

| Property        | Description                                                            | Type                                       | Required | Default  |
| --------------- | ---------------------------------------------------------------------- | ------------------------------------------ | -------- | -------- |
| `configuration` | Indicates the build configuration the product should be analyzed with. | [`ConfigurationName`](#configuration-name) | No       | `.debug` |

:::note Analyze Action Pre and Post Actions
Xcode does not support adding pre-actions or post-actions to Analyze actions. Analysis happens during building, so use the pre-actions and post-actions of the Build action instead.
:::

### Settings

A `Settings` object contains an optional dictionary with build settings and relative path to an `.xcconfig` file. It is initialized with the `.settings` static method passing the following attributes:

| Property          | Description                                                                      | Type                                        | Required | Default        |
| ----------------- | -------------------------------------------------------------------------------- | ------------------------------------------- | -------- | -------------- |
| `base`            | A dictionary with build settings that are inherited from all the configurations. | [`SettingsDictionary`](#settingsdictionary) | No       | `[:]`          |
| `debug`           | The debug configuration settings.                                                | [`Configuration`](#configuration)           | No       |                |
| `release`         | The release configuration settings.                                              | [`Configuration`](#configuration)           | No       |                |
| `defaultSettings` | An enum specifying the set of default settings.                                  | [`DefaultSettings`](#defaultsettings)       | No       | `.recommended` |

To specify multiple configurations beyond the default Debug and Release configuration _(e.g. Debug, Beta, Release)_, `Settings` can alternatively be initialized with the following attributes:

| Property          | Description                                                                      | Type                                             | Required | Default        |
| ----------------- | -------------------------------------------------------------------------------- | ------------------------------------------------ | -------- | -------------- |
| `base`            | A dictionary with build settings that are inherited from all the configurations. | [`SettingsDictionary`](#settingsdictionary)      | No       | `[:]`          |
| `configurations`  | A list of configurations.                                                 | [`[Configuration]`](#configuration) | Yes      |                |
| `defaultSettings` | An enum specifying the set of default settings.                                  | [`DefaultSettings`](#defaultsettings)            | No       | `.recommended` |

:::note Configurations
The list of configurations should have unique names and can't be empty. Please use the alternate settings initializer to leverage the default Debug & Release configurations.
:::

#### SettingsDictionary

`SettingsDictionary` is a typealias for the `[String: SettingValue]` dictionary, which is used to define a map of `xcodebuild` settings.

Besides the ability to define typed strings for these settings, Tuist has methods to avoid having typos when using strings for the keys. For example, instead of defining your dictionary this way:

```swift
let baseSettings: [String: SettingValue] = [
  "ENABLE_BITCODE": "YES",
  "SWIFT_COMPILATION_MODE": "wholemodule",
]
```

It can be written this way as well:

```swift
let baseSettings = SettingsDictionary()
    .bitcodeEnabled(true)
    .swiftCompilationMode(.wholemodule)
```

:::note Dictionary Literal
When using dictionary literals, it's necessary to set the type of the dictionary to `SettingsDictionary` to satisfy the compiler, as in `let mySettings: SettingsDictionary = [:]`
:::

Here is the list of functions provided by Tuist:

| Subject        | Function Signature                                                                                                   | Description                                                                                                                                                           |
| -------------- | -------------------------------------------------------------------------------------------------------------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| Code Signing   | `func manualCodeSigning(identity: String? = nil, provisioningProfileSpecifier: String? = nil) -> SettingsDictionary` | Sets `"CODE_SIGN_STYLE"` to `"Manual"`, `"CODE_SIGN_IDENTITY"` to `identity`, and `"PROVISIONING_PROFILE_SPECIFIER"` to `provisioningProfileSpecifier`                |
| Code signing   | `func automaticCodeSigning(devTeam: String) -> SettingsDictionary`                                                   | Sets `"CODE_SIGN_STYLE"` to `"Automatic"` and `"DEVELOPMENT_TEAM"` to `devTeam`                                                                                       |
| Code signing   | `func codeSignIdentityAppleDevelopment() -> SettingsDictionary`                                                      | Sets `"CODE_SIGN_IDENTITY"` to `"Apple Development"`                                                                                                                  |
| Code signing   | `func codeSignIdentity(_ identity: String) -> SettingsDictionary`                                                    | Sets `"CODE_SIGN_IDENTITY"` to `identity`                                                                                                                             |
| Versioning     | `func currentProjectVersion(_ version: String) -> SettingsDictionary`                                                | Sets `"CURRENT_PROJECT_VERSION"` to `version`                                                                                                                         |
| Versioning     | `func appleGenericVersioningSystem() -> SettingsDictionary`                                                          | Sets `"VERSIONING_SYSTEM"` to `"apple-generic"`                                                                                                                       |
| Versioning     | `func versionInfo(_ version: String, prefix: String? = nil, suffix: String? = nil) -> SettingsDictionary`            | Sets `"VERSION_INFO_PREFIX"` to `version`. If `prefix` and `suffix` are not `nil`, they\'re used as `"VERSION_INFO_PREFIX"` and `"VERSION_INFO_SUFFIX"` respectively. |
| Swift Settings | `func swiftVersion(_ version: String) -> SettingsDictionary`                                                         | Sets `"SWIFT_VERSION"` to `version`                                                                                                                                   |
| Swift Settings | `func otherSwiftFlags(_ flags: String...) -> SettingsDictionary`                                                     | Sets `"OTHER_SWIFT_FLAGS"` to `flags`                                                                                                                                 |
| Swift Settings | `func swiftCompilationMode(_ mode: SwiftCompilationMode) -> SettingsDictionary`                                      | Sets `"SWIFT_COMPILATION_MODE"` to the available `SwiftCompilationMode` (`"singlefile"` or `"wholemodule"`)                                                           |
| Swift Settings | `func swiftOptimizationLevel(_ level: SwiftOptimizationLevel) -> SettingsDictionary`                                 | Sets `"SWIFT_OPTIMIZATION_LEVEL"` to the available `SwiftOptimizationLevel` (`"-O"`, `"-Onone"` or `"-Osize"`)                                                        |
| Bitcode        | `func bitcodeEnabled(_ enabled: Bool) -> SettingsDictionary`                                                         | Sets `"ENABLE_BITCODE"` to `"YES"` or `"NO"`                                                                                                                          |

Notice that you don't depend exclusively on Tuist's built-in functions.
You can create your own `SettingsDictionary` extension methods and add them to your
`Project description helpers` this way:

```swift
public extension SettingsDictionary {
  func setSomeBuildSetting(_ value: String) -> SettingsDictionary {
      merging(["SOME_BUILD_SETTING": SettingValue(stringLiteral: value)])
  }
}
```

### Configuration

A `Configuration` object describes the build settings and the `.xcconfig` file of a project or target. It is initialized with either the `.debug` or `.release` static method passing the following attributes:

| Property   | Description                                    | Type                                        | Required | Default |
| ---------- | ---------------------------------------------- | ------------------------------------------- | -------- | ------- |
| `name` | The name of the configuration. | [`ConfigurationName`](#configuration-name) | Yes | |
| `settings` | A dictionary that contains the build settings. | [`SettingsDictionary`](#settingsdictionary) | No       | `[:]`   |
| `xcconfig`     | The path to the xcconfig file.                 | [`Path`](#path)                             | No       |         |

:::note Default build settings
Depending on the variant, debug or release, there are some build settings et by default. For example, `.debug` configuration would get `SWIFT_OPTIMIZATION_LEVEL = -Onone` whereas a `.release` would get `SWIFT_OPTIMIZATION_LEVEL = -Owholemodule`. Those can be overriden through the `settings` attribute.
:::

#### SettingValue

A `SettingValue` can be one of the following options:

| Case               | Description                                                          |
| ------------------ | -------------------------------------------------------------------- |
| `.string(String)`  | A single setting value for example "DEBUG=1".                        |
| `.array([String])` | Multi-value setting value for example `["$(inherited)", "DEBUG=1"]`. |

It conforms the `ExpressibleByStringLiteral` and `ExpressibleByArrayLiteral` protocols and therefore it can be initialized with a string or an array directly.

### FileElement

A `FileElement` can be one of the following options:

| Case                           | Description                                                                                                                         |
| ------------------------------ | ----------------------------------------------------------------------------------------------------------------------------------- |
| `.glob(pattern: Path)`         | A file path (or glob pattern) to include. For convenience, a string literal can be used as an alternate way to specify this option. |
| `.folderReference(path: Path)` | A directory path to include as a folder reference.                                                                                  |

## Resource Synthesizer

A `ResourceSynthesizer` creates accessors, so you can use your resources in a type-safe way. To read more about them, head over to [resource documentation](guides/resources.md).

### ResourceFileElements

List of files to include in the resources build phase.

| Property    | Description     | Type                                            | Required | Default |
| ----------- | --------------- | ----------------------------------------------- | -------- | ------- |
| `resources` | Resources list. | [`[ResourceFileElement]`](#resourcefileelement) | Yes      | `[]`    |

It conforms the `ExpressibleByStringLiteral` and `ExpressibleByArrayLiteral` protocols and therefore it can be initialized with a string or an array directly.

### ResourceFileElement

A `ResourceFileElement` can be one of the following options:

| Case                                           | Description                                                                                                                                           |
| ---------------------------------------------- | ----------------------------------------------------------------------------------------------------------------------------------------------------- |
| `.glob(pattern: Path, excluding: [Path], tags: [String])`         | A file path (or glob pattern) to include, a list of file paths (or glob patterns) to exclude, and ODR tags list. For convenience, a string literal can be used as an alternate way to specify this option. |
| `.folderReference(path: Path, tags: [String])` | A directory path to include as a folder reference and ODR tags list.                                                                                  |

### DefaultSettings

A `DefaultSettings` can be one of the following options:

| Case                                   | Description                                                                                                                                                                                                                                                           |
| -------------------------------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `.recommended(excluding: Set<String>)` | Recommended settings including warning flags to help you catch some of the bugs at the early stage of development. If you need to override certain settings in a `Configuration` it's possible to add those keys to `excluding`.                                |
| `.essential(excluding: Set<String>)`   | A minimal set of settings to make the project compile without any additional settings for example `'PRODUCT_NAME'` or `'TARGETED_DEVICE_FAMILY'`. If you need to override certain settings in a `Configuration` it's possible to add those keys to `excluding`. |
| `.none`                                | Tuist won't generate any build settings for the target or project.                                                                                                                                                                                                    |

:::note Essential Settings
The .essential option can be used in the event all warnings are defined in an `xcconfig` file to prevent Tuist from overriding them at the project or target level.
:::

:::note None settings
Projects may fail to compile if some essential settings are missing. Use .none only if you are specifying all the necessary build settings manually or via xcconfig files.
:::

### Deployment Target

The `DeploymentTarget` model represents the minimum operating system version your product will support. It can have any of the following values:

| Case                                                     | Description                                                                |
| -------------------------------------------------------- | -------------------------------------------------------------------------- |
| `.iOS(targetVersion: String, devices: DeploymentDevice)` | The minimum iOS version and the list of devices your product will support. |
| `.macOS(targetVersion: String)`                          | The minimum macOS version your product will support.                       |

### Deployment Device

The `DeploymentDevice` model represents the device your product will support. It can have any of the following values:

| Case      | Description       |
| --------- | ----------------- |
| `.iphone` | An iPhone device. |
| `.ipad`   | An iPad device.   |
| `.mac`    | A mac device.     |

### Path

A path represents a path to a file, directory, or a group of files represented by a glob expression. Paths can be relative and absolute. We **discourage** using absolute paths because they create a dependency with the environment where they are defined.

| Case                             | Description                                                                                                                                                       |
| -------------------------------- | ----------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `.relativeToCurrentFile(String)` | Initialize a path that is relative to the file that defines the path.                                                                                             |
| `.relativeToManifest(String)`    | Initialize a path that is relative to the directory that contains the manifest file being loaded, for example the directory that contains the Project.swift file. |
| `.relativeToRoot(String)`        | Initialize a path that is relative to the closest directory that contains a Tuist or a .git directory.                                                            |

:::note ExpressibleByStringLiteral
The path can be initialized with a plain string. It's equivalent to using `.relativeToManifest.` except when the path starts with `//`, in which case the path is considered relative to the root directory.
:::

### SchemeLanguage

Language to use in target action. Can be initialized with `String` which should be a valid language code or used as pre-defined pseudo language.

| Pre-defined languages                  | Description                                           |
| -------------------------------------- | ----------------------------------------------------- |
| `doubleLengthPseudoLanguage`           | `IDELaunchSchemeLanguageDoubleLocalizedStrings`       |
| `rightToLeftPseudoLanguage`            | `IDELaunchSchemeLanguageRightToLeftLayoutDirection`   |
| `accentedPseudoLanguage`               | `IDELaunchSchemeLanguageAccentedLatin`                |
| `boundedStringPseudoLanguage`          | `IDELaunchSchemeLanguageBoundedString`                |
| `rightToLeftWithStringsPseudoLanguage` | `IDELaunchSchemeLanguageRLO`                          |

### Target Reference

A target within a specified project. The project is specified through the path and should contain the target name.

| Property | Description                             | Type            | Required | Default                    |
| -------- | --------------------------------------- | --------------- | -------- | -------------------------- |
| `path`   | Path to the target's project directory. | [`Path`](#path) | No       | Workspace or project path. |
| `name`   | Name of the target.                     | String          | Yes      |                            |

### File Header Template

Lets you define custom file header template for built-in Xcode templates, e.g. when you create new Swift file you can automatically have your custom define file header.

Tuist automatically performs several template transformations for you

- if your template starts with comment slashes (`//`) we remove them as they are added automatically by Xcode
- if your template doesn't start with comment and whitespace or newline, we add a space - otherwise your header would be glued to implicit comment slashes which you probably do not want
- if your template has trailing newline, we remove it as it is implicitly added by Xcode

| Case              | Description                                      |
| ----------------- | ------------------------------------------------ |
| `.file(Path)`     | Use file at given path as file header template.  |
| `.string(String)` | Use given inline string as file header template. |

:::note ExpressibleByStringInterpolation
File header template can be initialized with a plain string, it is equivalent to using `.string(...)`
:::

### FileCodeGen

The `FileCodeGen` enum represents the possible source file attributes for source code generation
`NOTE`: This is only applicable to a subset of file tyes (e.g. intent definitions and ML models)

| Case      | Description                           |
| --------- | ------------------------------------- |
| `public ` | Public codegen(`codegen`)             |
| `private` | Private codegen(`private_codegen`).   |
| `project` | Project codegen(`project_codegen`).   |
| `disabled`| No codegen(`no_codegen`).             |
